@@ -1,12 +1,13 @@
 package OpenInteract::Handler::Error;
 
-# $Id: Error.pm,v 1.7 2001/07/13 14:48:51 lachoy Exp $
+# $Id: Error.pm,v 1.8 2001/08/10 14:48:47 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :level );
+use Text::Wrap    qw();
 
 @OpenInteract::Handler::Error::ISA     = qw( OpenInteract::Handler::GenericDispatcher SPOPS::Secure );
-$OpenInteract::Handler::Error::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::Error::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::Error::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::Error::default_method    = 'listing';
@@ -112,6 +113,14 @@ sub show {
             $params->{error_msg} = "You do not have permission to view this error object.";
         }
     }
+
+    # Grab the user here since sometimes the user might be 'superuser'
+    # who cannot be seen by mere mortals. This throws an error if you
+    # do in the template [% error_user = err.user %] and rather than a
+    # TRY/CATCH block we'll just do it here.
+
+    $params->{error_user} = eval { $error->user };
+
     local $Text::Wrap::columns = 60;
     $error->{system_msg} = Text::Wrap::wrap( '', '', $error->{system_msg} );
     $error->{user_msg}   = Text::Wrap::wrap( '', '', $error->{user_msg} );
