@@ -3,37 +3,89 @@ package OpenInteract::Handler::User;
 # $Id: User.pm,v 1.4 2001/07/13 15:16:14 lachoy Exp $
 
 use strict;
+use Data::Dumper  qw( Dumper );
 use SPOPS::Secure qw( :level :scope );
 
 @OpenInteract::Handler::User::ISA     = qw( OpenInteract::Handler::GenericDispatcher  SPOPS::Secure );
 $OpenInteract::Handler::User::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::User::author            = 'chris@cwinters.com';
-$OpenInteract::Handler::User::default_method    = 'listing';
-@OpenInteract::Handler::User::forbidden_methods = ();
-%OpenInteract::Handler::User::security          = ( 
- listing => SEC_LEVEL_READ, edit   => SEC_LEVEL_WRITE,
- show    => SEC_LEVEL_READ, remove => SEC_LEVEL_WRITE,  
-);
+$OpenInteract::Handler::User::default_method    = 'search_form';
+@OpenInteract::Handler::User::forbidden_methods = qw( 
+      _search_params _search_params_ldap _search_params_dbi );
+%OpenInteract::Handler::User::security          = (
+      listing => SEC_LEVEL_READ, edit   => SEC_LEVEL_WRITE,
+      show    => SEC_LEVEL_READ, remove => SEC_LEVEL_WRITE );
 
 use constant MAIN_SCRIPT => '/User';
 
 
+sub search_form {
+    return OpenInteract::Request->instance->template->handler(
+                   {}, {}, { name => 'base_user::user_search_form' } );
+}
+
+
 sub listing {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { main_script => MAIN_SCRIPT, 
+    my $params = { main_script => MAIN_SCRIPT,
                    error_msg   => $p->{error_msg} };
-    $params->{user_iterator} = eval { $R->user->fetch_iterator({ order => 'login_name' }) };
+    my $fetch_params = $class->_search_params;
+    $R->DEBUG && $R->scrib( 1, "Searching for users with parameters\n",
+                               Dumper( $fetch_params ) );
+    $params->{user_iterator} = eval { $R->user->fetch_iterator( $fetch_params) };
     if ( $@ ) {
         OpenInteract::Error->set( SPOPS::Error->get );
-        $R->throw( { code => 404 } );
-        $params->{user_list} = [];
+        $R->throw({ code => 404 });
     }
     $R->{page}->{title} = 'User Listing';
-    return $R->template->handler( {}, $params, 
-                                  { db      => 'user_list', 
-                                    package => 'base_user' } );
+    return $R->template->handler( {}, $params,
+                                  { name => 'base_user::user_list' } );
+}
+
+sub _search_params {
+    my ( $class ) = @_;
+    my $R   = OpenInteract::Request->instance;
+    my $apr = $R->apache;
+    my @search_fields = qw( first_name last_name email login_name );
+    my %s = map { $_ => $apr->param( $_ ) } @search_fields;
+    my $user_class = $R->user;
+    return $class->_search_params_ldap( \%s ) if ( $user_class->isa( 'SPOPS::LDAP' ) );
+    return $class->_search_params_dbi( \%s )  if ( $user_class->isa( 'SPOPS::DBI' ) );
+    return ();
+}
+
+
+sub _search_params_dbi {
+    my ( $class, $search ) = @_;
+    my @where = ();
+    my @value = ();
+    foreach my $field ( keys %{ $search } ) {
+        next unless ( $search->{ $field } );
+        push @where, " $field LIKE ? ";
+        push @value, "%$search->{ $field }%";
+    }
+    return { order => 'login_name',
+             where => join( ' AND ', @where ),
+             value => \@value };
+}
+
+
+sub _search_params_ldap {
+    my ( $class, $search ) = @_;
+    my $R      = OpenInteract::Request->instance;
+    my $field_map = $R->user->CONFIG->{field_map} || {};
+    my @filter_chunk = ();
+    foreach my $field ( keys %{ $search } ) {
+        next unless ( $search->{ $field } );
+        my $ldap_prop = $field_map->{ $field };
+        push @filter_chunk, "($ldap_prop=*$search->{ $field }*)";
+    }
+    unless ( scalar @filter_chunk ) {
+        return {};
+    }
+    return { filter => '(&' . join( '', @filter_chunk ) . ')' };
 }
 
 
