@@ -1,11 +1,11 @@
 package OpenInteract::User;
 
-# $Id: User.pm,v 1.3 2001/07/13 15:16:14 lachoy Exp $
+# $Id: User.pm,v 1.4 2001/08/12 05:37:44 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :level :scope );
 
-$OpenInteract::User::VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::User::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
 $OpenInteract::User::crypt_password = undef;
 
 
@@ -25,7 +25,7 @@ sub make_public {
 
  # First find the public group
 
-    my $groups = eval { $R->group->fetch_group({ where => 'name = ?', 
+    my $groups = eval { $R->group->fetch_group({ where => 'name = ?',
                                                  value => [ 'public' ] }) };
     if ( my $public = $groups->[0] ) {
 
@@ -37,10 +37,10 @@ sub make_public {
             $OpenInteract::Error::user_msg = 'Cannot make user part of public group.';
             die $OpenInteract::Error::user_msg;
         }
-    
-        # Then ensure the public can see (for now) this user 
 
-        eval { $self->set_security({ scope    => SEC_SCOPE_GROUP, 
+        # Then ensure the public can see (for now) this user
+
+        eval { $self->set_security({ scope    => SEC_SCOPE_GROUP,
                                      scope_id => $public->{group_id},
                                      level    => SEC_LEVEL_READ }) };
         if ( $@ ) {
@@ -53,7 +53,7 @@ sub make_public {
 }
 
 
-sub full_name { return join ' ', $_[0]->{first_name}, $_[0]->{last_name}; }
+sub full_name { return join ' ', $_[0]->{first_name}, $_[0]->{last_name} }
 
 
 sub increment_login {
