@@ -8,7 +8,7 @@ CREATE TABLE sys_error (
  filename      varchar(60) not null,
  line          smallint not null,
  action        varchar(25) null,
- user_id       %%INCREMENT_TYPE%% null,
+ user_id       %%USERID_TYPE%% null,
  session_id    varchar(32) null,
  browser       varchar(75) null,
  referer       varchar(150) null,
